@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2013 by StyleShare, Inc.
+# Copyright (C) 2011-2014 by Hong Minhee <http://hongminhee.org/>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -28,29 +28,52 @@ authenticate via the specific `OAuth 2.0`_ service e.g. Facebook_, Google_.
 .. _Google: http://www.google.com/
 
 """
-import random
-import urllib
-import urllib2
-import urlparse
-import hmac
-import hashlib
 import base64
-import Cookie
+import binascii
 import cgi
+try:
+    import Cookie
+except ImportError:
+    from http import cookies as Cookie
+import hashlib
+import hmac
 try:
     import simplejson as json
 except ImportError:
     import json
+import numbers
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
+import random
+try:
+    import urllib2
+except ImportError:
+    from urllib import request as urllib2
+try:
+    import urlparse
+except ImportError:
+    from urllib import parse as urlparse
+    urlencode = urlparse.urlencode
+else:
+    from urllib import urlencode
 
-__author__ = 'Hong Minhee' # http://dahlia.kr/
-__email__ = 'dahlia' "@" 'stylesha.re'
+__author__ = 'Hong Minhee'  # http://hongminhee.org/
+__email__ = 'minhee' "@" 'dahlia.kr'
 __license__ = 'MIT License'
-__version__ = '0.1.4'
-__copyright__ = '2011-2013, StyleShare Inc.'
+__version__ = '0.2.0'
+__copyright__ = '2011-2014, Hong Minhee'
+
+__all__ = ('AccessToken', 'Client', 'GitHubService', 'GithubService',
+           'Service', 'WSGIMiddleware', 'github', 'google', 'facebook')
+
+
+# Python 3 compatibility
+try:
+    basestring
+except NameError:
+    basestring = str
 
 
 class Service(object):
@@ -91,6 +114,8 @@ class Service(object):
 
         :param access_token: a valid :class:`AccessToken`
 
+        .. versionadded:: 0.1.2
+
         """
         raise NotImplementedError(
             "This Service does not provide a username for REMOTE_USER")
@@ -103,6 +128,8 @@ class Service(object):
 
         :param access_token: a valid :class:`AccessToken`
 
+        .. versionadded:: 0.1.3
+
         """
         return True
 
@@ -110,7 +137,7 @@ class Service(object):
         """Makes a :class:`Client` for the service.
 
         :param client_id: a client id
-        :type client_id: :class:`basestring`, :class:`int`, :class:`long`
+        :type client_id: :class:`basestring`, :class:`numbers.Integral`
         :param client_secret: client secret key
         :type client_secret: :class:`basestring`
         :returns: a client for the service
@@ -130,6 +157,12 @@ class GitHubService(Service):
                          protected application.
     :type allowed_orgs: :class:`basestring`,
                         :class:`collections.Container` of :class:`basestring`
+
+    .. versionadded:: 0.1.3
+       The ``allowed_orgs`` option.
+
+    .. versionadded:: 0.1.2
+
     """
 
     def __init__(self, allowed_orgs=None):
@@ -149,8 +182,10 @@ class GitHubService(Service):
 
         :param access_token: a valid :class:`AccessToken`
 
+        .. versionadded:: 0.1.2
+
         """
-        response = access_token.get('https://api.github.com/user');
+        response = access_token.get('https://api.github.com/user')
         response = response.read()
         response = json.loads(response)
         # Copy useful data
@@ -166,6 +201,8 @@ class GitHubService(Service):
 
         :param access_token: a valid :class:`AccessToken`
 
+        .. versionadded:: 0.1.3
+
         """
         # if there is no list of allowed organizations, any authenticated user
         # is allowed.
@@ -192,9 +229,9 @@ class Client(object):
     :param service: service the client connects to
     :type servie: :class:`Service`
     :param client_id: client id
-    :type client_id: :class:`basestring`, :class:`int`, :class:`long`
+    :type client_id: :class:`basestring`, :class:`numbers.Integral`
     :param client_secret: client secret key
-    :type client_secret: :class:`basestring`
+    :type client_secret: :class:basestring`
     :param \*\*extra: additional arguments for authorization e.g.
                       ``scope='email,read_stream'``
 
@@ -216,7 +253,7 @@ class Client(object):
         if not isinstance(service, Service):
             raise TypeError('service must be a wsgioauth2.Service instance, '
                             'not ' + repr(service))
-        elif isinstance(client_id, (int, long)):
+        elif isinstance(client_id, numbers.Integral):
             client_id = str(client_id)
         elif not isinstance(client_id, basestring):
             raise TypeError('client_id must be a string, not ' +
@@ -248,7 +285,7 @@ class Client(object):
         if state is not None:
             query['state'] = state
         return '{0}?{1}'.format(self.service.authorize_endpoint,
-                                urllib.urlencode(query))
+                                urlencode(query))
 
     def load_username(self, access_token):
         """Load a username from the configured service suitable for the
@@ -258,6 +295,8 @@ class Client(object):
 
         :param access_token: a valid :class:`AccessToken`
 
+        .. versionadded:: 0.1.2
+
         """
         self.service.load_username(access_token)
 
@@ -282,12 +321,23 @@ class Client(object):
                 'redirect_uri': redirect_uri,
                 'grant_type': 'authorization_code'}
         u = urllib2.urlopen(self.service.access_token_endpoint,
-                            data=urllib.urlencode(form))
-        content_type = u.info().gettype()
+                            data=urlencode(form).encode('utf-8'))
+        m = u.info()
+        try:
+            # Python 2
+            content_type = m.gettype()
+        except AttributeError:
+            # Python 3
+            content_type = m.get_content_type()
         if content_type == 'application/json':
             data = json.load(u)
         else:
-            data = urlparse.parse_qs(u.read())
+            data = dict(
+                (k.decode('utf-8')
+                 if not isinstance(k, str) and isinstance(k, bytes)
+                 else k, v)
+                for k, v in urlparse.parse_qs(u.read()).items()
+            )
         u.close()
         return AccessToken(data)
 
@@ -357,7 +407,7 @@ class WSGIMiddleware(object):
     :param application: wsgi application
     :type application: callable object
     :param secret: secret key for generating HMAC signature
-    :type secret: :class:`basestring`
+    :type secret: :class:`bytes`
     :param path: path prefix used for callback. by default, a randomly
                  generated complex path is used
     :type path: :class:`basestring`
@@ -367,7 +417,7 @@ class WSGIMiddleware(object):
     :param set_remote_user: Set to True to set the REMOTE_USER environment
                             variable to the authenticated username (if supported
                             by the :class:`Service`)
-    :type set_remote_user: bool
+    :type set_remote_user: :class:`bool`
     :param forbidden_path: What path should be used to display the 403 Forbidden
                            page.  Any forbidden user will be redirected to this
                            path and a default 403 Forbidden page will be shown.
@@ -379,7 +429,7 @@ class WSGIMiddleware(object):
                                   generic Forbidden page will be generated. Set
                                   this to :const:`True` to pass the request
                                   through to the protected application.
-    :type forbidden_passthrough: bool
+    :type forbidden_passthrough: :class:`bool`
     :param login_path:  The base path under which login will be required. Any
                         URL starting with this path will trigger the OAuth2
                         process.  The default is '/', meaning that the entire
@@ -387,6 +437,15 @@ class WSGIMiddleware(object):
                         path see the :attr:`login_path` option.
     :type login_path: :class:`basestring`
 
+    .. versionadded:: 0.1.4
+       The ``login_path`` option.
+
+    .. versionadded:: 0.1.3
+       The ``forbidden_path`` and ``forbidden_passthrough`` options.
+
+    .. versionadded:: 0.1.2
+       The ``set_remote_user`` option.
+
     """
 
     #: (:class:`basestring`) The default name for :attr:`cookie`.
@@ -398,7 +457,7 @@ class WSGIMiddleware(object):
     #: (callable object) The wrapped WSGI application.
     application = None
 
-    #: (:class:`basestring`) The secret key for generating HMAC signature.
+    #: (:class:`bytes`) The secret key for generating HMAC signature.
     secret = None
 
     #: (:class:`basestring`) The path prefix for callback URL. It always
@@ -421,6 +480,8 @@ class WSGIMiddleware(object):
     #: Any URL starting with this path will trigger the OAuth2 process.  The
     #: default is '/', meaning that the entire application is protected.  To
     #: override the default path see the :attr:`login_path` option.
+    #:
+    #: .. versionadded:: 0.1.4
     login_path = None
 
     #: (:class:`basestring`) The cookie name to be used for maintaining
@@ -437,8 +498,8 @@ class WSGIMiddleware(object):
         if not callable(application):
             raise TypeError('application must be an WSGI compliant callable, '
                             'not ' + repr(application))
-        if not isinstance(secret, basestring):
-            raise TypeError('secret must be a string, not ' + repr(secret))
+        if not isinstance(secret, bytes):
+            raise TypeError('secret must be bytes, not ' + repr(secret))
         if not (path is None or isinstance(path, basestring)):
             raise TypeError('path must be a string, not ' + repr(path))
         if not (forbidden_path is None or
@@ -457,7 +518,7 @@ class WSGIMiddleware(object):
         if path is None:
             seq = ('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
                    'abcdefghijklmnopqrstuvwxyz_-.')
-            path = ''.join(random.choice(seq) for x in xrange(40))
+            path = ''.join(random.choice(seq) for x in range(40))
             path = '__{0}__'.format(path)
         self.path = '/{0}/'.format(path.strip('/'))
         if forbidden_path is None:
@@ -476,33 +537,45 @@ class WSGIMiddleware(object):
         self.cookie = cookie
         self.set_remote_user = set_remote_user
 
+    def sign(self, value):
+        """Generate signature of the given ``value``.
+
+        .. versionadded:: 0.2.0
+
+        """
+        if not isinstance(value, bytes):
+            raise TypeError('expected bytes, not ' + repr(value))
+        return hmac.new(self.secret, value, hashlib.sha1).hexdigest()
+
     def redirect(self, url, start_response, headers={}):
         h = {'Content-Type': 'text/html; charset=utf-8', 'Location': url}
         h.update(headers)
-        start_response('307 Temporary Redirect', h.items())
-        e_url = cgi.escape(url)
-        yield '<!DOCTYPE html>'
-        yield '<html><head><meta charset="utf-8">'
-        yield '<meta http-equiv="refresh" content="0; url='
+        start_response('307 Temporary Redirect', list(h.items()))
+        e_url = cgi.escape(url).encode('iso-8859-1')
+        yield b'<!DOCTYPE html>'
+        yield b'<html><head><meta charset="utf-8">'
+        yield b'<meta http-equiv="refresh" content="0; url='
         yield e_url
-        yield '"><title>Redirect to '
+        yield b'"><title>Redirect to '
         yield e_url
-        yield '</title></head><body><p>Redirect to <a href="'
+        yield b'</title></head><body><p>Redirect to <a href="'
         yield e_url
-        yield '">'
+        yield b'">'
         yield e_url
-        yield '</a>&hellip;</p></body></html>'
+        yield b'</a>&hellip;</p></body></html>'
 
     def forbidden(self, start_response):
         """Respond with an HTTP 403 Forbidden status."""
-        h = {'Content-Type': 'text/html; charset=utf-8'}
-        start_response('403 Forbidden', h.items())
-        yield '<!DOCTYPE html>'
-        yield '<html><head><meta charset="utf-8">'
-        yield '<title>Forbidden</title></head>'
-        yield '<body><p>403 Forbidden - '
-        yield 'Your account does not have access to the requested resource.'
-        yield '</p></body></html>'
+        h = [('Content-Type', 'text/html; charset=utf-8')]
+        start_response('403 Forbidden', h)
+        yield b'<!DOCTYPE html>'
+        yield b'<html><head><meta charset="utf-8">'
+        yield b'<title>Forbidden</title></head>'
+        yield b'<body><p>403 Forbidden - '
+        yield b'Your account does not have access to the requested resource.'
+        yield b'<pre>'
+        yield b'</pre>'
+        yield b'</p></body></html>'
 
     def __call__(self, environ, start_response):
         url = '{0}://{1}{2}'.format(environ.get('wsgi.url_scheme', 'http'),
@@ -516,14 +589,14 @@ class WSGIMiddleware(object):
         cookie_dict = Cookie.SimpleCookie()
         cookie_dict.load(environ.get('HTTP_COOKIE', ''))
         query_dict = urlparse.parse_qs(query_string)
-
-        if environ.get('PATH_INFO').startswith(self.forbidden_path):
+        path = environ['PATH_INFO']
+        if path.startswith(self.forbidden_path):
             if self.forbidden_passthrough:
                 # Pass the forbidden request through to the app
-                return self.application(environ, start_response);
+                return self.application(environ, start_response)
             return self.forbidden(start_response)
 
-        elif environ.get('PATH_INFO').startswith(self.path):
+        elif path.startswith(self.path):
             code = query_dict.get('code')
             if not code:
                 # No code in URL - forbidden
@@ -546,11 +619,11 @@ class WSGIMiddleware(object):
                 return self.redirect(forbidden_uri, start_response)
 
             session = pickle.dumps(access_token)
-            sig = hmac.new(self.secret, session, hashlib.sha1).hexdigest()
-            signed_session = '{0},{1}'.format(sig, session)
+            sig = self.sign(session)
+            signed_session = sig.encode('ascii') + b',' + session
             signed_session = base64.urlsafe_b64encode(signed_session)
             set_cookie = Cookie.SimpleCookie()
-            set_cookie[self.cookie] = signed_session
+            set_cookie[self.cookie] = signed_session.decode('ascii')
             set_cookie[self.cookie]['path'] = '/'
             if 'expires_in' in access_token:
                 expires_in = int(access_token['expires_in'])
@@ -559,16 +632,19 @@ class WSGIMiddleware(object):
             return self.redirect(query_dict.get('state', [''])[0],
                                  start_response,
                                  headers={'Set-Cookie': set_cookie})
-        elif environ.get('PATH_INFO').startswith(self.login_path):
+        elif path.startswith(self.login_path):
             if self.cookie in cookie_dict:
                 session = cookie_dict[self.cookie].value
-                session = base64.urlsafe_b64decode(session)
-                if ',' in session:
-                    sig, val = session.split(',', 1)
-                    if sig == hmac.new(self.secret, val, hashlib.sha1).hexdigest():
+                try:
+                    session = base64.urlsafe_b64decode(session)
+                except binascii.Error:
+                    session = b''
+                if b',' in session:
+                    sig, val = session.split(b',', 1)
+                    if sig.decode('ascii') == self.sign(val):
                         try:
                             session = pickle.loads(val)
-                        except pickle.UnpicklingError:
+                        except (pickle.UnpicklingError, ValueError):
                             session = None
                     else:
                         session = None
@@ -609,5 +685,7 @@ google = Service(
 
 #: (:class:`GitHubService`) The predefined service for GitHub__.
 #:
+#: .. versionadded:: 0.1.2
+#:
 #: __ https://github.com/
 github = GitHubService()
